import { Metadata, Http, DateHelper } from '@zeedhi/core';
import { Modal, ModalService, Form, DialogService, Select, TextInput, DateRange, Textarea, Dashboard, Container, Card, LoadingService } from '@zeedhi/common';
import { TekGrid, TekRestDatasource } from "@zeedhi/teknisa-components-common";
import { CustomAlertService } from '@/Services/CustomAlertService';
import { Utilities } from '@/Services/Utilities';
import { Filter } from '@/Services/Filter/Filter';
import { Ordination } from '@/Services/Ordination/Ordination';
import { ChartViewFactory } from '@/Services/ChartView/ChartViewFactory';
import { ZmartChartsApex } from '@/Services/Chats/ZmartChartsApex';
import { PeriodsService } from '@/Services/PeriodsService';
import { DialogServiceCustom } from '@/Services/DialogServiceCustom';
import { ExpressionInterpreter } from '@/Services/ExpressionInterpreter';
import { SkeletonMetadata } from '@/Services/SkeletonMetadata';

export class DashboardConfigurationController {
    public tabsHeader: any = '';
    public dashboards: any = '';
    public currentCard: any = '';
    public currentCardId: any = '';
    public widget: any = '';
    public viewTypeId: any;
    public dashboardCards: any;
    public tabIndex: any = '';
    public filterReloadDashboard: any = '';
    public getLastImportationDate: any = '';
    public dashboardData: any;
    public counterPeriod: number = 0;
    public dashboardTitleText: string = '';
    public idDashboard: any = '';
    public idIndicator: any = '';
    public indicators: any = '';
    public conditionString: any = '';
    public previousOrPost: any = '';
    public dataSourceColumns: any = '';
    public rows: any = '';
    public color: string = '#FFFFFFF';
    private viewRowLogModal!: Modal;
    private indicatorDetailModal!: Modal;
    private dashboardLogModal!: Modal;
    private dashboardIndicatorsModal!: Modal;
    private dashboardConfigurationModal!: Modal;
    public gridName = "dashboardConfigurationGrid";
    public containerDashboard!: Container;
    protected user: any;
    periodFilterDashboard: any[] = [];
    periodFilter: any[] = [];

    // Início Métodos TekGrid Configurações de Dashboards (dashboardConfiguration.json)
    public get isDashboardVisible() {
        return this.masterDetailControl.dashboard;
    }

    public masterDetailControl: any = {
        grid: true,
        dashboard: false,
    };

    public showDashboard() {
        this.showMasterDetail("dashboard");
    }

    public showGrid() {
        this.showMasterDetail("grid");
    }

    public get isGridVisible() {
        return this.masterDetailControl.grid;
    }

    public showMasterDetail(option: string) {
        Object.keys(this.masterDetailControl).forEach((key: string) => { this.masterDetailControl[key] = key === option; });
    }

    public closeDetailsDashboard() {
        if (this.isDashboardVisible) {
            this.showGrid();
        } else {
            this.dashboardConfigurationModal.hide();
        }
    }

    public showDeleteDialog({ event }: any) {
        event.preventDefault();
        DialogService.show({
            name: 'delete-dialog',
            type: 'warning',
            title: ' ',
            text: 'Deseja remover o(s) registro(s) selecionado(s)?',
            buttons: [
                { name: 'não', component: 'ZdButton', label: 'NÃO', events: this.dialogEvents, outline: true },
                { name: 'sim', component: 'ZdButton', label: 'SIM', events: this.dialogEvents },
            ],
        });
    }

    //  Metodos de dialog
    public dialogEvents: any = {
        click: async ({ component }: any) => {
            if (component.name === "sim") {
                await this.deleteRow();
            }
            DialogService.hide();
        }
    };

    public async deleteRow() {
        const grid = Metadata.getInstance<TekGrid>(this.gridName);
        const selectedRows = grid.selectedRows;
        try {
            await Http.post("/view/remove", {
                "requestType": "DataSet",
                "dataset": selectedRows
            });
            CustomAlertService.deleteSuccessAlert();
            grid.selectedRows = [];
        } catch (err) {
            console.error(err);
            CustomAlertService.errorAlert("Ocorreu um erro.");
        }
        this.reload();
    }

    public async reload() {
        const grid = Metadata.getInstance<TekGrid>(this.gridName);
        const res = await grid.datasource.get();
    }

    public async openDashboardConfigurationModal({ event, component, row }: any) {
        // Checa se o componente que chamou a classe é um botão de edição
        if (component.name === "editDetailsDashboardButton") {
            row = this.dashboardData;
        }
        //  Checa se o evento foi de fato expandir a linha ao invés de click na mesma
        if (event.target.type === "button" || component.editing) {
            return;
        }
        this.dashboardConfigurationModal?.destroy();
        await this.createDashboardConfigurationModal();
        this.modalData = { type: "D", headerColor: "", headerGradient: "", ...row };
        if (row) {
            Object.keys(row).filter((key) => key && key.startsWith("tree__")).forEach((key) => delete this.modalData[key]);
        }
        this.dashboardConfigurationModal.show();
    }

    public async gridOnMounted() {
        const grid = Metadata.getInstance<TekGrid>(this.gridName);
        await this.reload();
        Utilities.getLoggedUser().then((res) => {
            const login = res;
            this.user = {
                userId: login.userId,
                username: login.username
            };
        });
    }

    public cancelInlineEdit() {
        Metadata.getInstance<TekGrid>(this.gridName).cancelEditedRows();
    }

    public async saveInlineEdit() {
        const grid = Metadata.getInstance<TekGrid>(this.gridName);
        const rows = grid.getEditedRows();
        rows.forEach((row: any) => { row.changed = true; row.order = Number(row.order); });
        try {
            const res = await Http.post("view/save", {
                "requestType": "DataSet",
                "disableLoader": false,
                "dataset": rows
            });
            grid.saveEditedRows();
            CustomAlertService.saveSuccessAlert();
        } catch (err) {
            CustomAlertService.errorAlert("Ocorreu um erro.");
            console.error(err);
        }
    }

    public switchToDashboard({ component, row }: any) {
        if (component.editing === true) return;
        this.dashboardData = row;
        this.idDashboard = '';
        this.idDashboard = row._id;
        this.rows = row;
        this.dashboardTitleText = this.dashboardData?.name;
        this.loadDashboard();
        this.showDashboard();
    }
    // Final Métodos TekGrid Configurações de Dashboards (dashboardConfiguration.json)
    // Inicio Métodos Modal Configurações de Dashboards (dashboardConfiguration.json)
    //  Metodos de Modal
    public async createDashboardConfigurationModal() {
        this.dashboardConfigurationModal = await ModalService.createFromJson('dashboardConfiguration/dashboardConfigurationModal/dashboardConfigurationModal', true);
    }

    public closeDashboardConfigurationModal() {
        this.dashboardConfigurationModal?.hide();
    }

    //  DetailForm
    public modalData: any;
    public get currentRowId() {
        return this.modalData._id;
    }
    public get modalTitle() {
        return (this.currentRowId ? "Editar" : "Adicionar") + " Dashboard";
    }
    public get isVisible() {
        return (this.currentRowId);
    }

    public get accordionIsVisible() {
        const dashboardTemplate = Metadata.getInstance<Select>('dashboardTemplate').value;
        return dashboardTemplate === 'accordion';
    }

    public async formOnMounted({ component }: any) {
        const form = Metadata.getInstance<Form>('detailForm');
        const data = this.modalData;
        form.value = data;
        Metadata.getInstance<TextInput>('headerBackgroundImg').value = data.headerBackgroundImg;
        Metadata.getInstance<TextInput>('headerBackgroundIcon').value = data.headerBackgroundIcon;
        Metadata.getInstance<TextInput>('headerColor').value = data.headerColor;
        Metadata.getInstance<TextInput>('headerGradient').value = data.headerGradient;
    }

    public async saveDashboardConfigurationModal() {
        const form = Metadata.getInstance<Form>('detailForm');
        const formAccordion = Metadata.getInstance<Form>('form-accordionProperties');
        form.value.created_by_name = this.currentRowId ? form.value.created_by_name : this.user.username;
        form.value.created_by = this.currentRowId ? form.value.created_by : this.user.userId;
        form.value.order = Number(form.value.order);
        if (!form.validate()) {
            return;
        }
        const data = {
            'dataset': [
                {
                    ...form.value,
                    ...formAccordion.value,
                    "indicators": [],
                    "__createdLocal": true,
                    "__is_new": true,
                    "changed": true,
                    "type": "D"
                }
            ],
            "disableLoader": false,
            "requestType": 'DataSet'
        };
        try {
            await Http.post('/view/save', data);
            CustomAlertService.saveSuccessAlert();
        } catch (err) {
            CustomAlertService.errorAlert("Ocorreu um erro.");
            console.error(err);
        }
        this.reload();
        this.closeDashboardConfigurationModal();
    }

    public createIdForIndicators() {
        if (this.dashboards.data.dataset.mainDashboard[0].indicators === undefined) {
            return;
        }
        this.dashboards.data.dataset.mainDashboard[0].indicators.forEach((indicator: any, index: any) => {
            const randomNumber = Math.random().toString(10).substring(2, 15);
            indicator.idIndicator = randomNumber;
            this.dashboards.data.dataset.mainDashboard.indicators[index].idIndicator = randomNumber;
            this.dashboardData.indicators[index].idIndicator = randomNumber;
        });
    }

    public skelletonIsVisible = true;

    get skelletonNotVisible() {
        return !this.skelletonIsVisible;
    }

    public async loadDashboard() {
        this.skelletonIsVisible = true;
        this.filterReloadDashboard = '';
        this.containerDashboard = Metadata.getInstance<Container>('containerDashboard');
        const buildMainDashboards = await Filter.dashboardFilter(this.dashboardTitleText, this.idDashboard);
        this.tabsHeader = '';
        this.tabsHeader = await buildMainDashboards.data.dataset.mainDashboard;
        this.dashboards = buildMainDashboards;
        this.containerDashboard.children[0].zmartSelected = true;
        if (this.dashboards.data.dataset.mainDashboard[0].indicators.length === 0) {
            this.containerDashboard.children[0].cards = [];
            this.skelletonIsVisible = false;
            return;
        }
        await this.createIdForIndicators();
        await this.dashboardsTabs(this.dashboards.data.dataset.mainDashboard.indicators, 0);
        setTimeout(() => {
            this.skelletonIsVisible = false;
        }, 1000);
    }

    public async getCurrentCardId({ component }: any, event: any) {
        // const dashboard = Metadata.getInstance<Dashboard>("dashTab" + this.num);
        const dashboard = Metadata.getInstance<Dashboard>("Dashboard");
        this.currentCardId = dashboard.currentCardId;
        if (component.name.includes('addIndicatorInDashboard')) {
            this.currentCardId = undefined;
            await this.openIndicatorDetailModal(this.currentCardId);
        } else if (component.name.includes('cardFooterEditButton')) {
            await this.openIndicatorDetailModal(this.currentCardId);
        } else if (component.name.includes('cardFooterDeleteButton')) {
            await this.showDeleteDialogIndicator();
        }
    }

    public async dashboardsTabs(dashboards: any, tabIndex: any, filter: boolean = false) {
        // const tab = Metadata.getInstance<Tabs>('tabsDashboard');
        if (this.containerDashboard.children[0].zmartSelectedFilter === false) {
            const indicators: any[] = [];
            const dashboardIndicatorsIds: any[] = [];
            // const containterDashboard = Metadata.getInstance<Container>('containerDashboard');
            // const tabName = Metadata.getInstance<Tabs>("tabsDashboard");
            this.containerDashboard.children[0].cards = [];

            if (this.filterReloadDashboard !== '') {
                dashboards.forEach(async (dashboard: any) => {
                    dashboard.parameters.period.start = this.periodFilterDashboard[0];
                    dashboard.parameters.period.end = this.periodFilterDashboard[1];
                });
            }
            dashboards.forEach(function (dashboard: any) {
                if (!dashboardIndicatorsIds.includes(dashboard.indicator)) {
                    dashboardIndicatorsIds.push(dashboard.indicator);
                }
            });

            if (dashboards[0] !== undefined) {
                this.getLastImportationDate = "";
                this.getLastImportationDate = await Filter.filterGetLastImportationDates(dashboardIndicatorsIds, this.tabsHeader[tabIndex].name);
            }
            Ordination.sort(dashboards); // Ordenando os indicadores
            console.log(dashboards);
            let widthCounter = 0;
            let heightCounter = 0;
            let biggestHeight = 0;
            setTimeout(async () => {
                this.viewTypeId = {};
                this.dashboardCards = {};
                dashboards.forEach(async (dashboard: any) => {
                    if (widthCounter + dashboard.width > 18) {
                        heightCounter += biggestHeight;
                        widthCounter = 0;
                        biggestHeight = 0;
                    }
                    this.dashboardCards["override_" + dashboard._id] = dashboard;
                    if (!dashboard.width) {
                        dashboard.width = 3;
                    }
                    if (!dashboard.height) {
                        dashboard.height = 3;
                    }
                    indicators.push(
                        {
                            "i": dashboard._id,
                            "cardId": dashboard.idIndicator,
                            "idIndicator": dashboard.idIndicator,
                            "path": "dashboardConfiguration/dashboards2",
                            "local": true,
                            "overrideNamedProps": {
                                "NAME": "override_" + dashboard._id,
                            },
                            "h": dashboard.height,
                            "w": dashboard.width,
                            "x": (dashboard.x === undefined) ? widthCounter : dashboard.x,
                            "y": (dashboard.y === undefined) ? heightCounter : dashboard.y,
                        },
                    );
                    widthCounter += dashboard.width;
                    if (biggestHeight === 0 || biggestHeight < dashboard.height) {
                        biggestHeight = dashboard.height;
                    }
                });
                this.containerDashboard.children[0].cards = indicators;
            }, 1000);
        }
        if (filter === true) {
            this.containerDashboard.children[0].zmartSelected = true;
            this.containerDashboard.children[0].zmartSelectedFilter = true;
        }
    }

    public buildSkeletonChart(component: any) {
        component.children = [
            SkeletonMetadata.skeletonDashboardIndicator().children[0]
        ];
    }

    public async onMountedChildrenDashboard({ component }: any) {
        await this.buildSkeletonChart(component);
        const dashboard = this.dashboardCards[component.name];
        const dashboardName = Metadata.getInstance<Dashboard>('Dashboard');
        const buildWidget = await Filter.filterBuildWidgetByViewIdNext(dashboard.parameters.period, dashboard, "");
        this.widget = '';
        this.widget = buildWidget.data.dataset.widget;
        this.viewTypeId[this.widget[0].id] = dashboard.viewTypeId;
        const chartType = ChartViewFactory.getInstanceChartType(this.viewTypeId[this.widget[0].id]);
        this.widget = chartType.getWidgetDashboard(this.widget, dashboard);
        const charts = ZmartChartsApex.build(this.widget, this, component);
        const widgetChats = charts.getCharts();
        widgetChats[0].grid = {
            "sm": "12",
            "lg": "12",
            "cols": "12",
            "md": "12",
        };
        widgetChats[0].height = this.heightCard(dashboard.height - 1) + "px";
        const componentDataImportation = this.periodLabel(dashboard);

        widgetChats.unshift(componentDataImportation); // Setando na primeira posição do Json
        if (this.widget[0].name === 'notFound') {
            component.children = this.widget;
        } else {
            component.children = widgetChats;
        }
    }

    public periodLabel(dashboard: any) {
        let componentDataImportation = {};
        const updateDate = this.getLastImportationDate.data.dataset.lastImportDates[dashboard.dataSourceId];
        if (dashboard.parameters.period.periodLabel === undefined) {
            componentDataImportation = {
                "name": "callDescriptionValue",
                "component": "ZdText",
                "cssClass": "zd-display-inline-flex",
                "cssStyle": "font-size: 11px; color: #aaa; line-height: 15px; font: Courier, monospace;",
                "text": dashboard.parameters.period.start + " - " + dashboard.parameters.period.end + " * Atualizado em : " + updateDate
            };
        } else {
            componentDataImportation = {
                "name": "callDescriptionValue",
                "component": "ZdText",
                "cssClass": "zd-display-inline-flex",
                "cssStyle": "font-size: 11px; color: #aaa; line-height: 15px; font: Courier, monospace;",
                "text": dashboard.parameters.period.periodLabel + " * Atualizado em : " + updateDate
            };
        }
        return componentDataImportation;
    }

    public heightCard(height: any) {
        return height * 130;
    }

    public convertHeightCard(height: any) {
        return Math.round(parseFloat(height) / 130);
    }

    public async generatePeriodHelperOptions(period: any) {
        const periodDate = await Http.post("/periods");
        const data = periodDate.data.data;
        if (period.helperOptions && period.helperOptions.length > 0) {
            period.helperOptions.forEach((option: any) => DateHelper.unregister(option.periodsCode));
        }
        data.forEach((option: any) => DateHelper.register(option.periodsCode, option.name, () => {
            const service = new PeriodsService();
            const periods = service.decodePeriod(option.periodsCode);
            return [periods.start, periods.end].map((date: any) => date.split('/').reverse().join('-')) as any;
        }));
        period.helperOptions = data.map((option: any) => option.periodsCode).filter((val: any) => val);
        period.helperValue = data.periodsCode;
        period.updateHelperHint();
    }

    public async createDashboardLogModal() {
        this.dashboardLogModal = await ModalService.createFromJson('dashboardConfiguration/dashboardLogModal/dashboardLogModal', true);
    }

    public async openDashboardLogModal({ event, component, row}: any) {
        this.dashboardLogModal?.destroy();
        await this.createDashboardLogModal();
        this.modalData = { type: "M", ...row};
        if (row) {
            Object.keys(row).filter((key) => key && key.startsWith("tree__")).forEach((key) => delete this.modalData[key]);
        }
        this.dashboardLogModal.show();
    }

    public closeDashboardLogModal() {
        this.dashboardLogModal?.hide();
    }

    public async setFilterDashboardLogModal() {
        const setFilter = Metadata.getInstance<TekGrid>('dashboardLogTekGrid');
          if (!(setFilter.datasource instanceof TekRestDatasource)) return;

          setFilter.datasource.dynamicFilter = {
              collectionId: [
                {
                  operation: "EQUALS",
                  relation: "AND",
                  value: this.idDashboard,
                }
              ],
              collectionName: [
                {
                  operation: "IN",
                  relation: "AND",
                  value: ["dashboard"],
                }
              ]
          };

          setFilter.datasource.get();
    }

    public async createDashboardIndicatorsModal() {
        this.dashboardIndicatorsModal = await ModalService.createFromJson('dashboardConfiguration/dashboardIndicatorsModal/dashboardIndicatorsModal', true);
    }

    public async createIndicatorsConfigurationModal({ event, component, row}: any) {
        const id = this.idDashboard;
        const dashboard = Metadata.getInstance<TekGrid>('dashboardConfigurationGrid');
        const indicatorDashboard = Metadata.getInstance<TekGrid>('dashboardIndicatorsTekGrid');
        dashboard.datasource.data.forEach(function (rowDashboard: any) {
            if (rowDashboard._id === id) {
                indicatorDashboard.datasource.data = rowDashboard.indicators;
            }
        });
    }

    public async openIndicatorsConfigurationModal({ event, component, row}: any) {
        this.dashboardIndicatorsModal?.destroy();
        await this.createDashboardIndicatorsModal();
        this.modalData = { type: "M", ...row};
        if (row) {
            Object.keys(row).filter((key) => key && key.startsWith("tree__")).forEach((key) => delete this.modalData[key]);
        }
        this.dashboardIndicatorsModal.show();
    }

    public closeIndicatorsConfigurationModal() {
        this.dashboardIndicatorsModal?.hide();
    }

    public async displayLogModal() {
        this.viewRowLogModal = await ModalService.createFromJson('dashboardConfiguration/dashboardLogModal/dashboardLogDetailsModal', true);
        this.viewRowLogModal.show();
    }

    public async viewRowLog({row}: any) {
        await this.displayLogModal();
        setTimeout(() => {
            const form = Metadata.getInstance<Form>('dashboardLogDetailsForm');
            form.value = row;
            const valueAfter = JSON.stringify(row.VALUE_AFTER);
            form.value.VALUE_AFTER = valueAfter;
        }, 100);
    }

    public viewRowLogClose() {
        this.viewRowLogModal?.destroy();
        this.viewRowLogModal.hide();
    }

    public async displayIndicatorDetailModal() {
        this.indicatorDetailModal = await ModalService.createFromJson('dashboardConfiguration/dashboardIndicatorsModal/dashboardIndicatorsDetailsModal', true);
        await this.indicatorDetailModal.show();
    }

    public async setOrderByValue(row: any) {
        const select = Metadata.getInstance<Select>('orderBy');
        await select.updateData(row.dataSourceColumns);
        if (row !== undefined) {
            const value = row.orderBy;
            select.setValue(value);
        }
    }

    public async setRowPeriod(row: any) {
        const formIndicatorsPeriods = Metadata.getInstance<Form>('dashboardIndicatorsPeriodsForm');
        for (let index = 0; index < row.period.length; index += 1) {
            if (index > 0) {
                this.newPeriod();
            }
            setTimeout(() => {
                formIndicatorsPeriods.getChildInstance<DateRange>('period_' + index).setDateValue(row.period[index].start + ' até ' + row.period[index].end);
            }, 100);
        }
    }

    protected editModal: any;
    public async openIndicatorDetailModal(currentCardId: any) {
        this.currentCard = undefined;
        if (this.dashboards.data.dataset.mainDashboard[0].indicators !== undefined) {
            this.dashboards.data.dataset.mainDashboard[0].indicators.forEach((indicator: any) => {
                if (indicator.idIndicator === currentCardId) {
                    this.currentCard = indicator;
                }
            });
        }
        await this.verifyIfEditModal(this.currentCard);
        if (this.currentCard !== undefined) {
            this.color = this.currentCard.color;
        } else {
            this.color = "#ffffff";
        }
        await this.displayIndicatorDetailModal();
        if (this.currentCard === undefined) {
            return;
        }
        const formIndicatorsDetail = Metadata.getInstance<Form>('dashboardIndicatorsDetailsForm');
        const formIndicatorsConditions = Metadata.getInstance<Form>('dashboardIndicatorsConditionsForm');
        const formIndicatorsPeriods = Metadata.getInstance<Form>('dashboardIndicatorsPeriodsForm');
        formIndicatorsDetail.value = this.currentCard;
        formIndicatorsConditions.value = this.currentCard;
        formIndicatorsPeriods.value = this.currentCard;
        await this.setOrderByValue(this.currentCard);
        await this.setRowPeriod(this.currentCard);
        await this.changeIndicatorField(this.currentCard);
    }

    public async verifyIfEditModal(currentCard: any) {
        if (currentCard === undefined || currentCard === "") {
            this.editModal = false;
        } else {
            this.editModal = true;
        }
    }

    public get modalFormIndicatorTitle() {
        return (this.editModal === true ? "Editar" : "Adicionar") + " Indicador";
    }

    public indicatorDetailModalClose() {
        this.indicatorDetailModal?.destroy();
        this.indicatorDetailModal.hide();
    }

    public async changeIndicatorField({row}: any) {
        const form = Metadata.getInstance<Form>('dashboardIndicatorsDetailsForm');
        const indicator = Metadata.getInstance<Select>('indicator');
        if (!indicator.selectedData) {
            form.getChildInstance<Select>('widget').disabled = true;
            form.getChildInstance<Select>('orderBy').disabled = true;
            form.getChildInstance<Select>('orderByDirection').disabled = true;
        } else {
            form.getChildInstance<Select>('widget').disabled = false;
            row = form.value;
            this.idIndicator = row.indicator;
            const setFilter = Metadata.getInstance<Select>('widget');
            if (!(setFilter.datasource instanceof TekRestDatasource)) return;
                setFilter.datasource.dynamicFilter = {
                    indicator: [
                        {
                            operation: "EQUALS",
                            relation: "AND",
                            value: this.idIndicator,
                        }
                    ]
            };
            await setFilter.datasource.get();
            await this.setOrderByValue(row);
            await this.changeWidgetField(row);
            await this.changeOrderByField();
        }
    }

    public async changeWidgetField(row: any) {
        const form = Metadata.getInstance<Form>('dashboardIndicatorsDetailsForm');
        const widget = form.getChildInstance<Select>('widget');
        const orderBy = form.getChildInstance<Select>('orderBy');
        if (widget.selectedData || widget.selectedData?.columns) {
            form.getChildInstance<Select>('orderBy').disabled = false;
            orderBy.datasource.data = widget.selectedData?.columns || row?.dataSourceColumns;
            await this.changeOrderByField();
        } else {
            form.getChildInstance<Select>('orderBy').disabled = true;
            form.getChildInstance<Select>('orderByDirection').disabled = true;
        }
    }

    public async changeOrderByField() {
        const form = Metadata.getInstance<Form>('dashboardIndicatorsDetailsForm');
        const orderBy = form.getChildInstance<Select>('orderBy');
        if (orderBy.selectedData) {
            form.getChildInstance<Select>('orderByDirection').disabled = false;
        } else {
            form.getChildInstance<Select>('orderByDirection').disabled = true;
        }
    }

    public async getPreviousConditionsString(component: any) {
        const form = Metadata.getInstance<Form>('dashboardIndicatorsConditionsForm');
        const indicator = Metadata.getInstance<Select>('indicator');
        if (!indicator.selectedData) {
            return DialogServiceCustom.showError('Não é possível validar a expressão se não houver um indicador selecionado.');
        }
        this.dataSourceColumns = indicator.selectedData.dataSourceColumns ? indicator.selectedData.dataSourceColumns : form.value.dataSourceColumns;
        this.conditionString = form.value.previousConditionsString;
        this.previousOrPost = "previous";
        return this.readComparisonExpression(this.dataSourceColumns, this.conditionString, this.previousOrPost, component);
    }

    public async getPostConditionsString(component: any) {
        const form = Metadata.getInstance<Form>('dashboardIndicatorsConditionsForm');
        const indicator = Metadata.getInstance<Select>('indicator');
        if (!indicator.selectedData) {
            return DialogServiceCustom.showError('Não é possível validar a expressão se não houver um indicador selecionado.');
        }
        this.dataSourceColumns = indicator.selectedData.dataSourceColumns ? indicator.selectedData.dataSourceColumns : form.value.dataSourceColumns;
        this.conditionString = form.value.postConditionsString;
        this.previousOrPost = "post";
        return this.readComparisonExpression(this.dataSourceColumns, this.conditionString, this.previousOrPost, component);
    }

    public async getPostGroupConditionsString(component: any) {
        const form = Metadata.getInstance<Form>('dashboardIndicatorsConditionsForm');
        const indicator = Metadata.getInstance<Select>('indicator');
        if (!indicator.selectedData) {
            return DialogServiceCustom.showError('Não é possível validar a expressão se não houver um indicador selecionado.');
        }
        this.dataSourceColumns = indicator.selectedData.dataSourceColumns ? indicator.selectedData.dataSourceColumns : form.value.dataSourceColumns;
        this.conditionString = form.value.postGroupConditionsString;
        this.previousOrPost = "postGroup";
        return this.readComparisonExpression(this.dataSourceColumns, this.conditionString, this.previousOrPost, component);
    }

    public readComparisonExpression(dataSourceColumns: any, conditionString: any, previousOrPost: any, component: any) {
        try {
			let expression = null;
			if (conditionString) {
				const columnsNames = dataSourceColumns.map(function (column: { name: any; }) {
					return column.name;
				});
				expression = ExpressionInterpreter.readComparisonExpression(conditionString, previousOrPost, columnsNames, columnsNames);
                if (component.name !== "saveModalButton") {
                    DialogServiceCustom.showSuccess('Expressão Válida');
                }
			}
			return expression;
		} catch (e) {
            if (component.name !== "saveModalButton") {
                return DialogServiceCustom.showError('Expressão inválida');
            }
            return null;
		}
    }

    public async newPeriod() {
        const form = Metadata.getInstance<Form>('dashboardIndicatorsPeriodsForm');
        // Esta condição é para o formulário de adição ou formulário de edição com 0 ou 1 períodos setados
        if (form.children.length === 2) {
            this.counterPeriod = 1;
        }
        const render = this.createNewPeriod(this.counterPeriod);
        form.children = form.children.concat(render);
        this.counterPeriod += 1;
    }

    public removePeriod(params: any) {
        const form = Metadata.getInstance<Form>('dashboardIndicatorsPeriodsForm');
        const clickedButtonGeneratePeriod = Number(params.component.name.split("_").pop());
        for (let index = 0; index < form.children.length; index += 1) {
            if (form.children[index].generate_Period === clickedButtonGeneratePeriod) {
                const periodName = form.children[index + 1].name;
                form.children.splice(index, 2);
                delete form.value[periodName];
            }
        }
    }

    public createNewPeriod(n: number): any {
        return [
                {
                    name: "btnTrash_" + n,
                    generate_Period: n,
                    component: "ZdButton",
                    icon: true,
                    small: true,
                    iconName: "mdi-trash-can-outline",
                    disabled: false,
                    cssClass: "zd-mt-4",
                    events: {
                        click: "{{DashboardConfigurationController.removePeriod}}"
                    },
                    grid: {
                        "cols": 1
                    }
                },
                {
                    name: "period_" + n,
                    generate_Period: n,
                    label: "Período (Inicio - Final)",
                    component: "ZdDateRange",
                    splitter: " até ",
                    dateFormat: "DD/MM/YYYY",
                    inputFormat: "DD/MM/YYYY",
                    displayFormat: "DD/MM/YYYY",
                    operation: "BETWEEN",
                    grid: {
                        "cols": 11
                    }
                }
        ];
    }

    public async createFormObject(formIndicatorsDetail: Form, formIndicatorsConditions: Form) {
        const name = formIndicatorsDetail.getChildInstance<TextInput>('name').value;
        const order = Number(formIndicatorsDetail.getChildInstance<TextInput>('order').value);
        const indicator = formIndicatorsDetail.getChildInstance<Select>('indicator');
        const widget = formIndicatorsDetail.getChildInstance<Select>('widget');
        const orderBy = formIndicatorsDetail.getChildInstance<Select>('orderBy');
        const orderByDirection = formIndicatorsDetail.getChildInstance<Select>('orderByDirection').value;
        const width = Number(formIndicatorsDetail.getChildInstance<TextInput>('width').value);
        const height = Number(formIndicatorsDetail.getChildInstance<TextInput>('height').value);
        const limitrows = Number(formIndicatorsDetail.getChildInstance<TextInput>('limitrows').value);
        const color = formIndicatorsDetail.getChildInstance<TextInput>('color').value;
        const actionOnClick = formIndicatorsDetail.getChildInstance<Select>('actionOnClick').value;
        const previousConditionsString = formIndicatorsConditions.getChildInstance<Textarea>('previousConditionsString').value;
        const postConditionsString = formIndicatorsConditions.getChildInstance<Textarea>('postConditionsString').value;
        const postGroupConditionsString = formIndicatorsConditions.getChildInstance<Textarea>('postGroupConditionsString').value;

        const formObject = {
            name,
            order,
            indicator: indicator.value,
            widget: widget.value,
            orderBy: orderBy.value,
            orderByDirection,
            width,
            height,
            limitrows,
            color,
            actionOnClick,
            previousConditionsString,
            postConditionsString,
            postGroupConditionsString,
            dataSourceColumns: indicator.selectValue?.dataSourceColumns,
            dataSource: indicator.selectValue?.dataSource,
            indicator_label: indicator.selectValue?.name,
            widget_label: widget.selectValue?.label,
            orderBy_label: orderBy.selectValue?.label,
            label: orderBy.selectValue?.label
        };

        return formObject;
    }

    public async createPeriodObject(formIndicatorsPeriods: Form) {
        const period: {}[] = [];
        let periodsLength = Object.keys(formIndicatorsPeriods.value).filter((key) => key && key.startsWith('period_')).length;

        for (let index = 0; index < periodsLength; index += 1) {
            if (formIndicatorsPeriods.value['period_' + index] === null) {
                index += 1;
            }
            if (formIndicatorsPeriods.value['period_' + index] === undefined) {
                periodsLength += 1;
            } else {
                period.push(
                    {
                        "end": formIndicatorsPeriods.value['period_' + index][1],
                        "start": formIndicatorsPeriods.value['period_' + index][0],
                    }
                );
            }
        }
        return period;
    }

    public async saveIndicators({component}: any) {
        const formIndicatorsDetail = Metadata.getInstance<Form>('dashboardIndicatorsDetailsForm');
        const formIndicatorsConditions = Metadata.getInstance<Form>('dashboardIndicatorsConditionsForm');
        const formIndicatorsPeriods = Metadata.getInstance<Form>('dashboardIndicatorsPeriodsForm');
        if (!formIndicatorsDetail.validate()) {
            return;
        }
        const formObject = await this.createFormObject(formIndicatorsDetail, formIndicatorsConditions);
        const periods = await this.createPeriodObject(formIndicatorsPeriods);
        const previousConditions = await this.getPreviousConditionsString(component);
        const postConditions = await this.getPostConditionsString(component);
        const postGroupConditions = await this.getPostGroupConditionsString(component);
        const concatForm = {
            ...formObject,
            previousConditions,
            postConditions,
            postGroupConditions,
            period: periods,
        };
        if (formIndicatorsDetail.value.idIndicator === undefined) {
            this.dashboardData.indicators[this.dashboardData.indicators.length] = concatForm;
        } else {
            this.dashboardData.indicators.forEach((indicator:any, index: number) => {
                if (indicator.idIndicator === formIndicatorsDetail.value.idIndicator) {
                    this.dashboardData.indicators[index] = concatForm;
                }
            });
        }
        this.dashboardData.indicators.forEach((indicator:any) => {
            delete indicator.idIndicator;
        });
        await this.saveData(this.dashboardData, 'save');
        this.indicatorDetailModalClose();
        this.loadDashboard();
    }

    public dialogEventsIndicator: any = {
        click: async ({ component }: any) => {
            if (component.name === "sim") {
                await this.deleteIndicator();
            }
            DialogService.hide();
        }
    };

    public showDeleteDialogIndicator() {
        DialogService.show({
            name: 'delete-dialog',
            type: 'warning',
            title: ' ',
            text: 'Deseja remover o indicador? Está ação não poderá ser desfeita!',
            buttons: [
                { name: 'não', component: 'ZdButton', label: 'NÃO', events: this.dialogEventsIndicator, outline: true },
                { name: 'sim', component: 'ZdButton', label: 'SIM', events: this.dialogEventsIndicator },
            ],
        });
    }

    public dialogEventsEditDashboard: any = {
        click: async ({ component }: any) => {
            if (component.name === "sim") {
                LoadingService.show("");
                await this.saveDashboard();
                LoadingService.hide();
            }
            DialogService.hide();
        }
    };

    public showEditDialogDashboard() {
        DialogService.show({
            name: 'edit-dialog',
            type: 'warning',
            title: ' ',
            text: 'Tem certeza que gostaria de salvar?',
            buttons: [
                { name: 'não', component: 'ZdButton', label: 'NÃO', outline: true, events: this.dialogEventsEditDashboard },
                { name: 'sim', component: 'ZdButton', label: 'SIM', events: this.dialogEventsEditDashboard },
            ],
        });
    }

    public async saveData(dashboardData: any, type: any) {
        const data = {
            'dataset': [
                {
                    ...dashboardData,
                    "__createdLocal": true,
                    "__is_new": false,
                    "changed": true,
                    "type": "D"
                }
            ],
            "disableLoader": false,
            "requestType": 'DataSet'
        };
        try {
            await Http.post('/view/save', data);
            if (type === 'save') {
                CustomAlertService.saveSuccessAlert();
            } else if (type === 'delete') {
                CustomAlertService.deleteSuccessAlert();
            }
        } catch (err) {
            CustomAlertService.errorAlert("Ocorreu um erro.");
            console.error(err);
        }
    }

    public async saveDashboard() {
        const dashboard = Metadata.getInstance<Dashboard>('Dashboard');
        await this.dashboardData.indicators.forEach((indicator: any, index: number) => {
            dashboard.cards.forEach((card: any, mark: number) => {
                if (card.idIndicator === indicator.idIndicator) {
                    this.dashboardData.indicators[index].order = mark + 1;
                    this.dashboardData.indicators[index].height = card.h;
                    this.dashboardData.indicators[index].width = card.w;
                    this.dashboardData.indicators[index].x = card.x;
                    this.dashboardData.indicators[index].y = card.y;
                    this.dashboardData.indicators[index]._id = card.i;
                }
            });
        });
        await this.dashboardData.indicators.forEach((indicator:any) => {
            delete indicator.idIndicator;
        });
        await this.saveData(this.dashboardData, 'save');
    }

    public async deleteIndicator() {
        await this.dashboardData.indicators.forEach((indicator:any, index: number) => {
            if (indicator.idIndicator === this.currentCardId) {
                this.dashboardData.indicators.splice(index, 1);
            }
        });
        await this.saveData(this.dashboardData, 'delete');
        DialogService.hide();
        this.loadDashboard();
    }
}
